import { OnDestroy } from '@angular/core';
/**
 * A callback invoked when a store value changes. It is called with the latest value of a given store.
 */
export declare type SubscriberFunction<T> = (value: T) => void;
/**
 * A partial [observer](https://github.com/tc39/proposal-observable#api) notified when a store value changes. A store will call the `next` method every time the store's state is changing.
 */
export interface SubscriberObject<T> {
    next: SubscriberFunction<T>;
    invalidate: () => void;
}
/**
 * Expresses interest in store value changes over time. It can be either:
 * - a callback function: {@link SubscriberFunction};
 * - a partial observer: {@link SubscriberObject}.
 */
export declare type Subscriber<T> = SubscriberFunction<T> | Partial<SubscriberObject<T>> | null | undefined;
/**
 * A function to unsubscribe from value change notifications.
 */
export declare type UnsubscribeFunction = () => void;
/**
 * An object with the `unsubscribe` method.
 * Subscribable stores might choose to return such object instead of directly returning {@link UnsubscribeFunction} from a subscription call.
 */
export interface UnsubscribeObject {
    /**
     * A method that acts as the {@link UnsubscribeFunction}.
     */
    unsubscribe: UnsubscribeFunction;
}
export declare type Unsubscriber = UnsubscribeObject | UnsubscribeFunction;
/**
 * Represents a store accepting registrations (subscribers) and "pushing" notifications on each and every store value change.
 */
export interface SubscribableStore<T> {
    /**
     * A method that makes it possible to register "interest" in store value changes over time.
     * It is called each and every time the store's value changes.
     * A registered subscriber is notified synchronously with the latest store value.
     *
     * @param subscriber - a subscriber in a form of a {@link SubscriberFunction} or a {@link SubscriberObject}. Returns a {@link Unsubscriber} (function or object with the `unsubscribe` method) that can be used to unregister and stop receiving notifications of store value changes.
     * @returns The {@link UnsubscribeFunction} or {@link UnsubscribeObject} that can be used to unsubscribe (stop state change notifications).
     */
    subscribe(subscriber: Subscriber<T>): Unsubscriber;
}
/**
 * This interface augments the base {@link SubscribableStore} interface with the Angular-specific `OnDestroy` callback. The {@link Readable} stores can be registered in the Angular DI container and will automatically discard all the subscription when a given store is destroyed.
 */
export interface Readable<T> extends SubscribableStore<T>, OnDestroy {
    subscribe(subscriber: Subscriber<T>): UnsubscribeFunction & UnsubscribeObject;
}
/**
 * A function that can be used to update store's value. This function is called with the current value and should return new store value.
 */
export declare type Updater<T> = (value: T) => T;
/**
 * Builds on top of {@link Readable} and represents a store that can be manipulated from "outside": anyone with a reference to writable store can either update or completely replace state of a given store.
 *
 * ```typescript
 * // reset counter's store value to 0 by using the {@link Writable.set} method
 * counterStore.set(0);
 *
 * // increment counter's store value by using the {@link Writable.update} method
 * counterStore.update(currentValue => currentValue + 1);
 * ```
 */
export interface Writable<T> extends Readable<T> {
    /**
     * Replaces store's state with the provided value.
     * @param value - value to be used as the new state of a store.
     */
    set(value: T): void;
    /**
     * Updates store's state by using an {@link Updater} function.
     * @param updater - a function that takes the current state as an argument and returns the new state.
     */
    update(updater: Updater<T>): void;
}
/**
 * A utility function to get the current value from a given store.
 * It works by subscribing to a store, capturing the value (synchronously) and unsubscribing just after.
 *
 * @param store - a store from which the current value is retrieved.
 *
 * @example
 * ```typescript
 * const myStore = writable(1);
 * console.log(get(myStore)); // logs 1
 * ```
 */
export declare function get<T>(store: SubscribableStore<T>): T;
/**
 * Base class that can be extended to easily create a custom {@link Readable} store.
 *
 * @example
 * ```typescript
 * class CounterStore extends Store {
 *    constructor() {
 *      super(1); // initial value
 *    }
 *
 *    reset() {
 *      this.set(0);
 *    }
 *
 *    increment() {
 *      this.update(value => value + 1);
 *    }
 * }
 *
 * const store = new CounterStore(1);
 *
 * // logs 1 (initial value) upon subscription
 * const unsubscribe = store.subscribe((value) => {
 *    console.log(value);
 * });
 * store.increment(); // logs 2
 * store.reset(); // logs 0
 *
 * unsubscribe(); // stops notifications and corresponding logging
 * ```
 */
export declare abstract class Store<T> implements Readable<T> {
    private _value;
    private _subscribers;
    private _cleanupFn;
    /**
     *
     * @param _value - Initial value of the store
     */
    constructor(_value: T);
    private _start;
    private _stop;
    /**
     * Replaces store's state with the provided value.
     * Equivalent of {@link Writable.set}, but internal to the store.
     *
     * @param value - value to be used as the new state of a store.
     */
    protected set(value: T): void;
    /**
     * Updates store's state by using an {@link Updater} function.
     * Equivalent of {@link Writable.update}, but internal to the store.
     *
     * @param updater - a function that takes the current state as an argument and returns the new state.
     */
    protected update(updater: Updater<T>): void;
    /**
     * Function called when the number of subscribers changes from 0 to 1
     * (but not called when the number of subscribers changes from 1 to 2, ...).
     * If a function is returned, it will be called when the number of subscribers changes from 1 to 0.
     *
     * @example
     *
     * ```typescript
     * class CustomStore extends Store {
     *    onUse() {
     *      console.log('Got the fist subscriber!');
     *      return () => {
     *        console.log('All subscribers are gone...');
     *      };
     *    }
     * }
     *
     * const store = new CustomStore();
     * const unsubscribe1 = store.subscribe(() => {}); // logs 'Got the fist subscriber!'
     * const unsubscribe2 = store.subscribe(() => {}); // nothing is logged as we've got one subscriber already
     * unsubscribe1(); // nothing is logged as we still have one subscriber
     * unsubscribe2(); // logs 'All subscribers are gone...'
     * ```
     */
    protected onUse(): Unsubscriber | void;
    /**
     * Default Implementation of the {@link SubscribableStore.subscribe}, not meant to be overridden.
     * @param subscriber - see {@link SubscribableStore.subscribe}
     */
    subscribe(subscriber: Subscriber<T>): UnsubscribeFunction & UnsubscribeObject;
    ngOnDestroy(): void;
}
export interface OnUseArgument<T> {
    (value: T): void;
    set: (value: T) => void;
    update: (updater: Updater<T>) => void;
}
/**
 * A convenience function to create {@link Readable} store instances.
 * @param value - Initial value of a readable store.
 * @param onUseFn - A function called when the number of subscribers changes from 0 to 1
 * (but not called when the number of subscribers changes from 1 to 2, ...).
 * If a function is returned, it will be called when the number of subscribers changes from 1 to 0.
 *
 * @example
 * ```typescript
 * const clock = readable("00:00", setState => {
 *   const intervalID = setInterval(() => {
 *     const date = new Date();
 *     setState(`${date.getHours()}:${date.getMinutes()}:${date.getSeconds()}`);
 *   }, 1000);
 *
 *   return () => clearInterval(intervalID);
 * });
 * ```
 */
export declare function readable<T>(value: T, onUseFn?: (arg: OnUseArgument<T>) => void | Unsubscriber): Readable<T>;
/**
 * A convenience function to create {@link Writable} store instances.
 * @param value - initial value of a new writable store.
 * @param onUseFn - A function called when the number of subscribers changes from 0 to 1
 * (but not called when the number of subscribers changes from 1 to 2, ...).
 * If a function is returned, it will be called when the number of subscribers changes from 1 to 0.
 *
 * @example
 * ```typescript
 * const x = writable(0);
 *
 * x.update(v => v + 1); // increment
 * x.set(0); // reset back to the default value
 * ```
 */
export declare function writable<T>(value: T, onUseFn?: (arg: OnUseArgument<T>) => void | Unsubscriber): Writable<T>;
declare type SubscribableStores = SubscribableStore<any> | readonly [SubscribableStore<any>, ...SubscribableStore<any>[]];
declare type SubscribableStoresValues<S> = S extends SubscribableStore<infer T> ? T : {
    [K in keyof S]: S[K] extends SubscribableStore<infer T> ? T : never;
};
declare type SyncDeriveFn<T, S> = (values: SubscribableStoresValues<S>) => T;
declare type AsyncDeriveFn<T, S> = (values: SubscribableStoresValues<S>, set: OnUseArgument<T>) => Unsubscriber | void;
export declare abstract class DerivedStore<T, S extends SubscribableStores = SubscribableStores> extends Store<T> {
    private _stores;
    constructor(_stores: S, initialValue: T);
    protected onUse(): Unsubscriber | void;
    protected abstract derive(values: SubscribableStoresValues<S>): Unsubscriber | void;
}
/**
 * A convenience function to create a new store with a state computed from the latest values of dependent stores.
 * Each time the state of one of the dependent stores changes, a provided derive function is called to compute a new, derived state.
 *
 * @param stores - a single store or an array of dependent stores
 * @param deriveFn - a function that is used to compute a new state based on the latest values of dependent stores
 *
 * @example
 * ```typescript
 * const x$ = writable(2);
 * const y$ = writable(3);
 * const sum$ = derived([x$, $y], ([x, y]) => x + y);
 *
 * // will log 5 upon subscription
 * sum$.subscribe((value) => {
 *    console.log(value)
 * });
 *
 * x$.set(3); // will re-evaluate the `([x, y]) => x + y` function and log 6 as this is the new state of the derived store
 * ```
 */
export declare function derived<T, S extends SubscribableStores>(stores: S, deriveFn: SyncDeriveFn<T, S>): Readable<T>;
export declare function derived<T, S extends SubscribableStores>(stores: S, deriveFn: AsyncDeriveFn<T, S>, initialValue: T): Readable<T>;
export {};
//# sourceMappingURL=index.d.ts.map