var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Injectable } from '@angular/core';
const symbolObservable = (typeof Symbol === 'function' && Symbol.observable) || '@@observable';
const noop = () => { };
const bind = (object, fnName) => {
    const fn = object ? object[fnName] : null;
    return typeof fn === 'function' ? fn.bind(object) : noop;
};
const toSubscriberObject = (subscriber) => typeof subscriber === 'function'
    ? { next: subscriber.bind(null), invalidate: noop }
    : { next: bind(subscriber, 'next'), invalidate: bind(subscriber, 'invalidate') };
const returnThis = function () {
    return this;
};
const asReadable = (store) => ({
    subscribe: store.subscribe.bind(store),
    ngOnDestroy: store.ngOnDestroy.bind(store),
    [symbolObservable]: returnThis,
});
const queue = [];
function processQueue() {
    for (const [subscriberFn, value] of queue) {
        subscriberFn(value);
    }
    queue.length = 0;
}
const callUnsubscribe = (unsubscribe) => typeof unsubscribe === 'function' ? unsubscribe() : unsubscribe.unsubscribe();
function notEqual(a, b) {
    const tOfA = typeof a;
    if (tOfA !== 'function' && tOfA !== 'object') {
        return !Object.is(a, b);
    }
    return true;
}
/**
 * A utility function to get the current value from a given store.
 * It works by subscribing to a store, capturing the value (synchronously) and unsubscribing just after.
 *
 * @param store - a store from which the current value is retrieved.
 *
 * @example
 * ```typescript
 * const myStore = writable(1);
 * console.log(get(myStore)); // logs 1
 * ```
 */
export function get(store) {
    let value;
    callUnsubscribe(store.subscribe((v) => (value = v)));
    return value;
}
/**
 * Base class that can be extended to easily create a custom {@link Readable} store.
 *
 * @example
 * ```typescript
 * class CounterStore extends Store {
 *    constructor() {
 *      super(1); // initial value
 *    }
 *
 *    reset() {
 *      this.set(0);
 *    }
 *
 *    increment() {
 *      this.update(value => value + 1);
 *    }
 * }
 *
 * const store = new CounterStore(1);
 *
 * // logs 1 (initial value) upon subscription
 * const unsubscribe = store.subscribe((value) => {
 *    console.log(value);
 * });
 * store.increment(); // logs 2
 * store.reset(); // logs 0
 *
 * unsubscribe(); // stops notifications and corresponding logging
 * ```
 */
let Store = class Store {
    /**
     *
     * @param _value - Initial value of the store
     */
    constructor(_value) {
        this._value = _value;
        this._subscribers = new Set();
        this._cleanupFn = null;
    }
    _start() {
        this._cleanupFn = this.onUse() || noop;
    }
    _stop() {
        const cleanupFn = this._cleanupFn;
        if (cleanupFn) {
            this._cleanupFn = null;
            callUnsubscribe(cleanupFn);
        }
    }
    /**
     * Replaces store's state with the provided value.
     * Equivalent of {@link Writable.set}, but internal to the store.
     *
     * @param value - value to be used as the new state of a store.
     */
    set(value) {
        if (notEqual(this._value, value)) {
            this._value = value;
            if (!this._cleanupFn) {
                // subscriber not yet initialized
                return;
            }
            const needsProcessQueue = queue.length == 0;
            for (const subscriber of this._subscribers) {
                subscriber.invalidate();
                queue.push([subscriber.next, value]);
            }
            if (needsProcessQueue) {
                processQueue();
            }
        }
    }
    /**
     * Updates store's state by using an {@link Updater} function.
     * Equivalent of {@link Writable.update}, but internal to the store.
     *
     * @param updater - a function that takes the current state as an argument and returns the new state.
     */
    update(updater) {
        this.set(updater(this._value));
    }
    /**
     * Function called when the number of subscribers changes from 0 to 1
     * (but not called when the number of subscribers changes from 1 to 2, ...).
     * If a function is returned, it will be called when the number of subscribers changes from 1 to 0.
     *
     * @example
     *
     * ```typescript
     * class CustomStore extends Store {
     *    onUse() {
     *      console.log('Got the fist subscriber!');
     *      return () => {
     *        console.log('All subscribers are gone...');
     *      };
     *    }
     * }
     *
     * const store = new CustomStore();
     * const unsubscribe1 = store.subscribe(() => {}); // logs 'Got the fist subscriber!'
     * const unsubscribe2 = store.subscribe(() => {}); // nothing is logged as we've got one subscriber already
     * unsubscribe1(); // nothing is logged as we still have one subscriber
     * unsubscribe2(); // logs 'All subscribers are gone...'
     * ```
     */
    onUse() { }
    /**
     * Default Implementation of the {@link SubscribableStore.subscribe}, not meant to be overridden.
     * @param subscriber - see {@link SubscribableStore.subscribe}
     */
    subscribe(subscriber) {
        const subscriberObject = toSubscriberObject(subscriber);
        this._subscribers.add(subscriberObject);
        if (this._subscribers.size == 1) {
            this._start();
        }
        subscriberObject.next(this._value);
        const unsubscribe = () => {
            const removed = this._subscribers.delete(subscriberObject);
            if (removed && this._subscribers.size === 0) {
                this._stop();
            }
        };
        unsubscribe.unsubscribe = unsubscribe;
        return unsubscribe;
    }
    ngOnDestroy() {
        const hasSubscribers = this._subscribers.size > 0;
        this._subscribers.clear();
        if (hasSubscribers) {
            this._stop();
        }
    }
    [symbolObservable]() {
        return this;
    }
};
Store = __decorate([
    Injectable()
], Store);
export { Store };
/**
 * A convenience function to create {@link Readable} store instances.
 * @param value - Initial value of a readable store.
 * @param onUseFn - A function called when the number of subscribers changes from 0 to 1
 * (but not called when the number of subscribers changes from 1 to 2, ...).
 * If a function is returned, it will be called when the number of subscribers changes from 1 to 0.
 *
 * @example
 * ```typescript
 * const clock = readable("00:00", setState => {
 *   const intervalID = setInterval(() => {
 *     const date = new Date();
 *     setState(`${date.getHours()}:${date.getMinutes()}:${date.getSeconds()}`);
 *   }, 1000);
 *
 *   return () => clearInterval(intervalID);
 * });
 * ```
 */
export function readable(value, onUseFn = noop) {
    const ReadableStoreWithOnUse = class extends Store {
        onUse() {
            const setFn = (v) => this.set(v);
            setFn.set = setFn;
            setFn.update = (updater) => this.update(updater);
            return onUseFn(setFn);
        }
    };
    return asReadable(new ReadableStoreWithOnUse(value));
}
let WritableStore = class WritableStore extends Store {
    constructor(value) {
        super(value);
    }
    set(value) {
        super.set(value);
    }
    update(updater) {
        super.update(updater);
    }
};
WritableStore = __decorate([
    Injectable()
], WritableStore);
/**
 * A convenience function to create {@link Writable} store instances.
 * @param value - initial value of a new writable store.
 * @param onUseFn - A function called when the number of subscribers changes from 0 to 1
 * (but not called when the number of subscribers changes from 1 to 2, ...).
 * If a function is returned, it will be called when the number of subscribers changes from 1 to 0.
 *
 * @example
 * ```typescript
 * const x = writable(0);
 *
 * x.update(v => v + 1); // increment
 * x.set(0); // reset back to the default value
 * ```
 */
export function writable(value, onUseFn = noop) {
    const WritableStoreWithOnUse = class extends WritableStore {
        onUse() {
            const setFn = (v) => this.set(v);
            setFn.set = setFn;
            setFn.update = (updater) => this.update(updater);
            return onUseFn(setFn);
        }
    };
    const store = new WritableStoreWithOnUse(value);
    return Object.assign(Object.assign({}, asReadable(store)), { set: store.set.bind(store), update: store.update.bind(store) });
}
function isSyncDeriveFn(fn) {
    return fn.length <= 1;
}
let DerivedStore = class DerivedStore extends Store {
    constructor(_stores, initialValue) {
        super(initialValue);
        this._stores = _stores;
    }
    onUse() {
        let initDone = false;
        let pending = 0;
        const stores = this._stores;
        const isArray = Array.isArray(stores);
        const storesArr = isArray
            ? stores
            : [stores];
        const dependantValues = new Array(storesArr.length);
        let cleanupFn = null;
        const callCleanup = () => {
            const fn = cleanupFn;
            if (fn) {
                cleanupFn = null;
                callUnsubscribe(fn);
            }
        };
        const callDerive = () => {
            if (initDone && !pending) {
                callCleanup();
                cleanupFn = this.derive(isArray ? dependantValues : dependantValues[0]) || noop;
            }
        };
        const unsubscribers = storesArr.map((store, idx) => store.subscribe({
            next: (v) => {
                dependantValues[idx] = v;
                pending &= ~(1 << idx);
                callDerive();
            },
            invalidate: () => {
                pending |= 1 << idx;
            },
        }));
        initDone = true;
        callDerive();
        return () => {
            callCleanup();
            unsubscribers.forEach(callUnsubscribe);
        };
    }
};
DerivedStore = __decorate([
    Injectable()
], DerivedStore);
export { DerivedStore };
export function derived(stores, deriveFn, initialValue) {
    const Derived = isSyncDeriveFn(deriveFn)
        ? class extends DerivedStore {
            derive(values) {
                this.set(deriveFn(values));
            }
        }
        : class extends DerivedStore {
            derive(values) {
                const setFn = (v) => this.set(v);
                setFn.set = setFn;
                setFn.update = (updater) => this.update(updater);
                return deriveFn(values, setFn);
            }
        };
    return asReadable(new Derived(stores, initialValue));
}
//# sourceMappingURL=index.js.map